/* 70-762 Lesson 11 Lab 1 - Creating and Configuring Memory-Optimized Tables */

--To enable an application to use In-Memory OLTP, complete the following tasks: 

--Create a memory-optimized data filegroup and add a container to the filegroup 
--Create memory-optimized tables and indexes 
--Load data into the memory-optimized table and update statistics after loading the data
--
--Next, create natively compiled stored procedures to access data in memory-optimized tables
--Can also use traditional, interpreted Transact-SQL to access data in memory-optimized tables
--As needed, migrate data from existing tables to memory-optimized tables 

/*>>>>STEP 1<<<<*/
--Create a directory c:\ImData
USE master;
GO

DROP DATABASE IF EXISTS Certimoltp;
GO

CREATE DATABASE Certimoltp;   
GO 

---------------------------------------------------------------------------------------
/*>>>>STEP 2<<<<*/ 
--Create a memory-optimized data filegroup and add a container to the filegroup 
--------------------------------------  
-- create database with a memory-optimized filegroup and a container.  
ALTER DATABASE Certimoltp ADD FILEGROUP Certimoltp_mod CONTAINS MEMORY_OPTIMIZED_DATA   
ALTER DATABASE Certimoltp ADD FILE (name='Certimoltp_mod1', filename='c:\Imdata\Certimoltp_mod1') TO FILEGROUP Certimoltp_mod   
ALTER DATABASE Certimoltp SET MEMORY_OPTIMIZED_ELEVATE_TO_SNAPSHOT=ON  
GO  

-----------------------------------------------------------------------------------------
/*>>>>STEP 3<<<<*/
--Create memory-optimized tables and indexes 
--Load data into the memory-optimized table and update statistics after loading the data

USE Certimoltp;  
GO

DROP TABLE IF EXISTS dbo.ShoppingCart;
GO

-- create a durable (data will be persisted) memory-optimized table  
-- two of the columns are indexed  
  CREATE TABLE dbo.ShoppingCart (   
    ShoppingCartId INT IDENTITY(1,1) PRIMARY KEY NONCLUSTERED,  
    UserId INT NOT NULL INDEX ix_UserId NONCLUSTERED HASH WITH (BUCKET_COUNT=1000000),   
    CreatedDate DATETIME2 NOT NULL,   
    TotalPrice MONEY  
    ) WITH (MEMORY_OPTIMIZED=ON)   
  GO  

--------------------------------------------------------------------------
DROP TABLE IF EXISTS dbo.UserSession;
GO
 -- create a non-durable table. Data will not be persisted, data loss if the server turns off unexpectedly  
 CREATE TABLE dbo.UserSession (   
   SessionId INT IDENTITY(1,1) PRIMARY KEY NONCLUSTERED HASH WITH (BUCKET_COUNT=400000),   
   UserId int NOT NULL,   
   CreatedDate DATETIME2 NOT NULL,  
   ShoppingCartId INT,  
   INDEX ix_UserId NONCLUSTERED HASH (UserId) WITH (BUCKET_COUNT=400000)   
 )   
 WITH (MEMORY_OPTIMIZED=ON, DURABILITY=SCHEMA_ONLY)   
 GO  

-- insert data into the tables  
INSERT dbo.UserSession VALUES (342, SYSDATETIME(), 4)   
INSERT dbo.UserSession VALUES (65, SYSDATETIME(), NULL)   
INSERT dbo.UserSession VALUES (8798, SYSDATETIME(), 1)   
INSERT dbo.UserSession VALUES (80, SYSDATETIME(), NULL)   
INSERT dbo.UserSession VALUES (4321, SYSDATETIME(), NULL)   
INSERT dbo.UserSession VALUES (8578, SYSDATETIME(), NULL)   

INSERT dbo.ShoppingCart VALUES (8798, SYSDATETIME(), NULL)   
INSERT dbo.ShoppingCart VALUES (23, SYSDATETIME(), 45.4)   
INSERT dbo.ShoppingCart VALUES (80, SYSDATETIME(), NULL)   
INSERT dbo.ShoppingCart VALUES (342, SYSDATETIME(), 65.4)   
GO  

-- verify table contents   
  SELECT * FROM dbo.UserSession   
  SELECT * FROM dbo.ShoppingCart   
  GO  

--  update statistics on memory-optimized tables   
  UPDATE STATISTICS dbo.UserSession WITH FULLSCAN, NORECOMPUTE   
  UPDATE STATISTICS dbo.ShoppingCart WITH FULLSCAN, NORECOMPUTE   
  GO  

-- in an explicit transaction, assign a cart to a session and update the total price.   
-- SELECT/UPDATE/DELETE statements in explicit transactions   
  BEGIN TRAN   
   UPDATE dbo.UserSession SET ShoppingCartId=3 WHERE SessionId=4   
   UPDATE dbo.ShoppingCart SET TotalPrice=65.84 WHERE ShoppingCartId=3   
 COMMIT;   
 GO   

 -- verify table contents   
 SELECT *   
 FROM dbo.UserSession u JOIN dbo.ShoppingCart s on u.ShoppingCartId=s.ShoppingCartId   
 WHERE u.SessionId=4   
 GO  

 ----------------------------------------------------------------------------------
 --Exercise continues in Lab 2
 ----------------------------------------------------------------------------------