/* 70-762 Lesson 13 Lab 4 - Monitoring Performance Using the Query Store */

/* Run the following queries to have some current data for the Query Store */

USE WideWorldImporters;
GO

---Hash Match (Aggregate)----------------
---Grouping rows contributes a large percentage to cost of query
---Notice the thickness of the arrows
---Consider minimizing performance impact by creating indexed view
---   to perform the aggregation or just minimizing rows to aggregate
SELECT
	YEAR(TransactionDate) AS 'Transaction Year',
	COUNT(SupplierTransactionID) AS 'Transaction Count'
FROM Purchasing.SupplierTransactions
GROUP BY YEAR(TransactionDate);
GO


---Hash Match (Inner Join)--------------
---Query uses two Hash Match (inner Join) operators
---Consider adding indexes to eliminate Hash Match operators
SELECT
	si.StockItemName,
	c.ColorName,
	s.SupplierName
FROM Warehouse.StockItems AS si
	JOIN Warehouse.Colors AS c
ON si.ColorID = c.ColorID
	JOIN Purchasing.Suppliers AS s
ON s.SupplierID = si.SupplierID;
GO

--Clustered Index Scan Operator and Clustered Index Seek Operator----
Use WideWorldImporters;
GO

SELECT s.StockItemID,
      s.StockItemName,
      s.SupplierID,
	  h.QuantityOnHand
FROM Warehouse.StockItems AS s
JOIN Warehouse.StockItemHoldings AS h
ON s.StockItemID = h.StockItemID
WHERE s.SupplierID = 12;
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

--Review Query Store Reports--
/*
Use Top Resource Consuming Queries
Use Tracked Queries
Compare Query Plans
*/

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--Using Query Store Views--------------
--The following query returns information
--on queries and plans in the query store
--Query text ID (45904) - modify to your own IDs

SELECT Txt.query_text_id, Txt.query_sql_text, Pl.plan_id, Qry.*  
FROM sys.query_store_plan AS Pl  
JOIN sys.query_store_query AS Qry  
    ON Pl.query_id = Qry.query_id  
JOIN sys.query_store_query_text AS Txt  
    ON Qry.query_text_id = Txt.query_text_id ;