/*70-762 Lesson 4.5 Lab 3 - Using Non-clustered Columnstore Indexes in OLTP Tables to Support Analytics */

Use WideWorldImporters;
GO

/**Real-time analytics uses an updateable columnstore index on rowstore table 
Columnstore index maintains copy of data 
OLTP and analytics workloads run against separate data copies **/
--WideWorldImporters.Sales.OrderLines table includes a nonclustered
--columnstore index 
--Use Ctrl-M to Include Actual Execution Plan-----------------------


--Reviewing Columns and Compression Delay of NCCI in Sales.Orderlines table-----------
/****** Object:  Index [NCCX_Sales_OrderLines] ******/
--CREATE NONCLUSTERED COLUMNSTORE INDEX [NCCX_Sales_OrderLines] ON [Sales].[OrderLines]
--(
--	[OrderID],
--	[StockItemID],
--	[Description],
--	[Quantity],
--	[UnitPrice],
--	[PickedQuantity]
--)WITH (DROP_EXISTING = OFF, COMPRESSION_DELAY = 0) ON [USERDATA]
--GO

----------------------------------------------------------------------
/**********Run next 2 queries as a batch*****************************/
Use WideWorldImporters;
GO

--Query uses Nonclustered Columnstore Index---
SELECT max(UnitPrice) AS 'Most Expensive Item Ordered' 
FROM Sales.OrderLines; 
GO

--Query uses Clustered B-tree Index--------
SELECT max(UnitPrice) AS 'Most Expensive Item Ordered' 
FROM Sales.OrderLines
option (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);
GO



----------------------------------------------------------------------
/**********Run next 2 queries as a batch*****************************/
Use WideWorldImporters;
GO

--Query uses Nonclustered Columnstore Index---
select StockItemID, 
	SUM(UnitPrice) AS 'Sum of Unit Price', 
	AVG(UnitPrice) AS 'Average Unit Price'
from Sales.OrderLines 
where UnitPrice > 90.0 and PickedQuantity=10
group by StockItemID;
GO
--Query uses Clustered B-tree Index--------
select StockItemID, 
	SUM(UnitPrice) AS 'Sum of Unit Price', 
	AVG(UnitPrice) AS 'Average Unit Price'
from Sales.OrderLines 
where UnitPrice > 90.0 and PickedQuantity=10
group by StockItemID
option (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);
GO

--Tax Rate is not included in the NCCI Index------
--Query reverts to using Clustered Index scan----
select StockItemID, 
	SUM(UnitPrice) AS 'Sum of Unit Price', 
	AVG(UnitPrice) AS 'Average Unit Price',
	AVG(TaxRate) AS 'Average Tax Rate'
from Sales.OrderLines 
where UnitPrice > 90.0 and PickedQuantity=10
group by StockItemID;
GO





----------------------------------------------------------------------------------------------
--------------EXTRA---------------------------------------------------------------------------
-- look at the rowgroups
--select object_name(object_id), index_id, row_group_id, delta_store_hobt_id, state_desc, total_rows, trim_reason_desc, transition_to_compressed_state_desc
--from sys.dm_db_column_store_row_group_physical_stats
--where object_id = object_id('Sales.OrderLines')

--select customerID, SUM(UnitPrice) AS 'Sum of Unit Price', AVG(UnitPrice) AS 'Average Unit Price'
--from Sales.OrderLines AS OL
--JOIN
--Sales.Orders AS O
--ON O.OrderID = OL.OrderID 
--where UnitPrice > 90.0 and PickedQuantity=10
--group by customerID;
