/*70-762 Lesson 5 Lab 2 - Creating and Managing Constraints */

/* 
Uncomment the following code block to 
create the CertLiveDB database, 
the Production schema within the CertLiveDB database, 
and the table Production.Products with Primary Key constraint
*/

/*>>>>>>>>>>>>>>>>>>BEGIN CODE BLOCK>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
USE master;
GO

--Delete the CertLiveDB database if it exists --
DROP DATABASE IF EXISTS CertLiveDB;


--Create and Use the CertLiveDB database --

CREATE DATABASE CertLiveDB;
GO
*/

USE CertLiveDB;
GO


--Create the Production schema in the CertLiveDB database --
--DROP SCHEMA if it exists - but remember-----
--SCHEMA must be empty to be dropped----------
--DROP SCHEMA IF EXISTS Production------------
--DROP SCHEMA IF EXISTS Production;
--GO

CREATE SCHEMA Production AUTHORIZATION dbo;
GO

--------------------------------------------------------------------------------
--WORKING WITH PRIMARY KEYS AND FOREIGN KEYS------------------------------------
--------------------------------------------------------------------------------
--Create the table Products in the Production schema of the CertLiveDB database --
DROP TABLE IF EXISTS Production.Products;
GO

--Define PRIMARY KEY Constraint on ProductID --
CREATE TABLE Production.Products
   (ProductID INT  NOT NULL,
    ProductName varchar(100) NOT NULL,
	CategoryID INT NOT NULL,
    Price money NULL,
    ProductDescription varchar(max) NULL
	CONSTRAINT PK_Products_ProductID PRIMARY KEY(ProductID),
	);
GO

--Create Production.Categories table --
DROP TABLE IF EXISTS Production.Categories;
GO

CREATE TABLE Production.Categories(
	CategoryID INT IDENTITY(1,1) NOT NULL,
	CategoryName NVARCHAR (100),
	Description NVARCHAR(max)
	CONSTRAINT PK_Categories_CategoryID PRIMARY KEY(CategoryID),
);
GO

--INSERT VALUES INTO PRODUCTS TABLE --
INSERT INTO Production.Products
	VALUES (761,'Querying Data with Transact SQL', 2, 165.00, 'Manage Data with Transact-SQL; Program Databases Using Transact-SQL'),
	(762,'Developing SQL Databases', 2, 165.00,'Design and Implement Database Objects; Implement Programmability Objects'),
	(764,'Administering SQL Database Infrastructure',1, 165.00,'Configure Database Access and Auditing; Manage Backup and Restore'),
	(765,'Administering a SQL Database Infrastructure', 1, 165.00,'Implement SQL in Azure; Manage Storage'),
	(695, 'Deploying Windows Desktops and Enterprise Applications', 3, 165.00, 'Implementing the Operating System Deployment Infrastructure'); 
GO   


--INSERT VALUES INTO CATEGORIES TABLE --
INSERT INTO Production.Categories (categoryname ,description)
	VALUES(N'SQL 2016 Database Administration', N'Microsoft Solutions Associate'),
	(N'SQL 2016 Database Development', N'Microsoft Certified Solutions Expert'),
	(N'Mobility', N'Microsoft Certified Solutions Expert');
GO

------------------------------------------------------------------------------------
--Create FOREIGN KEY on Production.Products ----------------------------------------
ALTER TABLE Production.Products WITH CHECK
	ADD CONSTRAINT FK_Products_Categories FOREIGN KEY(CategoryID)
	REFERENCES Production.Categories (CategoryID);
GO

SELECT *
FROM Production.Products;
GO

SELECT *
FROM Production.Categories;
GO



-- Create a JOIN of the Products and Categories tables------------------------------
--Using the key references----------------------------------------------------------
USE CertLiveDB;
GO

SELECT P.ProductName, C.CategoryName
 FROM Production.Products AS P
	JOIN Production.Categories AS C
		ON P.CategoryID = C.CategoryID
GO  

--------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------




-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--Adding DEFAULT, CHECK, UNIQUE, and FOREIGN KEY Constraints-----------------------
-----------------------------------------------------------------------------------

--Add constraints when Creating Table-------------------------


DROP TABLE IF EXISTS Production.ProductsNew;
GO

CREATE TABLE Production.ProductsNew
   (ProductID INT  NOT NULL,
    ProductName varchar(50) NOT NULL,
	CategoryID INT NOT NULL,
    Price money NOT NULL
	  CONSTRAINT DFT_ProductsNew_price DEFAULT(0),
    ProductDescription varchar(max) NULL
	CONSTRAINT PK_ProductsNew_ProductID PRIMARY KEY(ProductID),
	CONSTRAINT UQ_ProductsNEW_ProductName UNIQUE(ProductName),
	CONSTRAINT CHK_ProductNew_Price
	CHECK (Price>=0)
	);

--Add constraints to Existing Table-------------------------
---------------------------------------------------------------

-----------------------------------------------------------------------------------
--Add DEFAULT constraint to Production.Products----------------------------------
ALTER TABLE Production.Products
	ADD CONSTRAINT DFT_ProductsPrice
	DEFAULT (0) FOR Price;
GO

INSERT INTO Production.Products (ProductID, ProductName, CategoryID, ProductDescription)
	VALUES(800,'Into the Clouds We Go',3,'Programming in Flight');
GO

SELECT *
FROM Production.Products;
GO


--To DROP DEFAULT Constraint----------
--If modifying, re create the default constraint----

DROP DEFAULT IF EXISTS DFT_ProductsPrice;
GO
------------------------------------------------------------------------------------
--Add CHECK constraint to Production.Products ------------------------------------
ALTER TABLE Production.Products WITH CHECK
	ADD CONSTRAINT CHK_Products_Price
	CHECK (Price>=0);
GO

--INSERT a negative Price which conflicts with CHECK constraint---------------------------------
--Error message: The INSERT statement conflicted with the CHECK constraint "CHK_Products_Price"-
INSERT INTO Production.Products
	VALUES(500,'PowerShell for Fun',2,-40.00,'We pay you to Learn PowerShell');
GO

--Delete CHECK Constraint
ALTER TABLE Production.Products
	DROP CONSTRAINT CHK_Products_Price;
GO

---------------------------------------------------------------------------------------
--Add UNIQUE constraint to Production.Products-----------------------------------------
ALTER TABLE Production.Products
	ADD CONSTRAINT UQ_Products_ProductName UNIQUE(ProductName);
GO

INSERT INTO Production.Products
	VALUES(900,'Querying Data with Transact SQL',2,165.00,'Query Data with Advanced SQL Components');
GO


--Delete UNIQUE Constraint
ALTER TABLE Production.Products
	DROP CONSTRAINT UQ_Products_ProductName;
GO

---------------------------------------------------------------------------------------
--Add FOREIGN KEY constraint to Production.Products------------------------------------
ALTER TABLE Production.Products 
	ADD CONSTRAINT FK_Products_Categories FOREIGN KEY (CategoryID)     
    REFERENCES Production.Categories (CategoryID);     
GO     

--Delete FOREIGN KEY Constraint
ALTER TABLE Production.Products
	DROP CONSTRAINT FK_Products_Categories;
GO    

---------------------------------------------------------------------------------------
--Add PRIMARY KEY constraint to Production.Products-----------------------------------------
ALTER TABLE Production.Products
	ADD CONSTRAINT PK_Products_ProductID PRIMARY KEY(ProductID);
GO

--Delete PRIMARY KEY Constraint
ALTER TABLE Production.Products
	DROP CONSTRAINT PK_Products_ProductID;
GO

---------------------------------------------------------------------
/**********Listing CONSTRAINTS************************************/
--To List DEFAULT constraints--------
SELECT *
FROM sys.default_constraints;
GO

SELECT * 
FROM sys.objects 
WHERE type = 'D';
GO


--------------------------------------------
--To List CHECK Constraints-------------------
SELECT * 
FROM sys.objects 
WHERE type = 'C';
GO

--Lists CHECK constraints and definition--
SELECT * 
FROM sys.check_constraints; 
GO

--Includes the CHECK Clause----
SELECT * 
FROM INFORMATION_SCHEMA.CHECK_CONSTRAINTS; 
GO

-----------------------------------------------
--To List UNIQUE Constraints-------------------
SELECT *
FROM sys.key_constraints
WHERE type = 'UQ';
GO

SELECT * 
FROM sys.objects 
WHERE type = 'UQ';
GO

SELECT * 
FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS 
WHERE CONSTRAINT_TYPE='UNIQUE'

-----------------------------------------------
--To list PRIMARY KEY Constraints-----------
SELECT *
FROM sys.key_constraints
WHERE type = 'PK';
GO

SELECT * 
FROM sys.objects 
WHERE type = 'PK';
GO

--Displays Metadata information
SELECT * 
FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS 
WHERE CONSTRAINT_TYPE='PRIMARY KEY'

-------------------------------------------------------
--To List FOREIGN KEY Constraints-------
--Also states delete and update referential description
SELECT *
FROM sys.foreign_keys;
GO

SELECT *
FROM sys.foreign_key_columns;
GO


SELECT * 
FROM sys.objects 
WHERE type = 'F';
GO

---Displays Metadata information
SELECT * 
FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS 
WHERE CONSTRAINT_TYPE='FOREIGN KEY';
GO

SELECT * 
FROM INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS; 
GO

-------------------------------------------------------------------------