/*70-762 Lesson 5 Lab 4 - Creating and Managing PRIMARY KEY Constraints */

--/* 
--Uncomment the following code block to 
--create the CertLiveDB database, 
--the Production schema within the CertLiveDB database, 
--and the table Production.Products with Primary Key constraint
--*/

--/*>>>>>>>>>>>>>>>>>>BEGIN CODE BLOCK>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--USE master;
--GO

----Delete the CertLiveDB database if it exists --
--DROP DATABASE IF EXISTS CertLiveDB;


----Create and Use the CertLiveDB database --

--CREATE DATABASE CertLiveDB;
--GO
--*/

--USE CertLiveDB;
--GO


----Create the Production schema in the CertLiveDB database --
----DROP SCHEMA if it exists - but remember-----
----SCHEMA must be empty to be dropped----------
----DROP SCHEMA IF EXISTS Production------------
--DROP SCHEMA IF EXISTS Production;
--GO

--CREATE SCHEMA Production AUTHORIZATION dbo;
--GO

USE CertLiveDB;
GO

/*
>>>>>>>>>>>>>>>>>>Using SEQUENCE>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
A Sequence is preferred over an Identity property when:
A number needed before data inserted into the table
Single series of numbers needs to be shared between multiple tables or multiple columns within a table.
Series needs to be recycled when a specified number is reached
*/
CREATE SEQUENCE CountBy3
   AS tinyint
    START WITH 1
    INCREMENT BY 1
    MINVALUE 1
    MAXVALUE 3
    CYCLE ;
GO

SELECT NEXT VALUE FOR CountBy3 AS SurveyGroup, Name FROM sys.objects ;
GO

/*
>>>>>>>>>>>>>>>>>>>Creating a SEQUENCE used in a table>>>>>>>>>>
*/
USE CertLiveDB
GO

--Create the Cert schema
CREATE SCHEMA Sales;
GO

DROP TABLE IF EXISTS Sales.Orders;
GO

-- Create a table
CREATE TABLE Sales.Orders
    (OrderID int,
    Name varchar(20) NOT NULL,
    Qty int NOT NULL
	CONSTRAINT PK_Orders_OrderID PRIMARY KEY(OrderID),
	);
GO

-- Create a sequence
CREATE SEQUENCE Sales.CountBy1
    START WITH 1
    INCREMENT BY 1 ;
GO

-- Insert three records
INSERT Sales.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Sales.CountBy1, 'apple', 8) ;
INSERT Sales.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Sales.CountBy1, 'banana', 6) ;
INSERT Sales.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Sales.CountBy1, 'orange', 12) ;
GO

-- View the table
SELECT * FROM Sales.Orders ;
GO

--CLEANUP--
--Drop table Orders and drop schema Sales --
DROP TABLE IF EXISTS Sales.Orders;
GO
DROP SEQUENCE IF EXISTS Sales.CountBy1
GO
--DROP SCHEMA IF EXISTS Sales
--GO
-----------------------------------------------------------------
----------------------------------------------------------------
-- Creating a table using NEWID as the default value -----------
-- for the uniqueidentifier data type --------------------------
DROP TABLE IF EXISTS Sales.Customers;
GO

CREATE TABLE Sales.Customers
(
 CustomerID uniqueidentifier NOT NULL
   DEFAULT NEWID(),
 Company varchar(30) NOT NULL,
 ContactName varchar(60) NOT NULL, 
 CountryRegion varchar(20) NOT NULL, 
);
GO

--------------------------------------------------------------------------
--Assign the default value of NEWID() to each new and existing row -------
--This ensures a unique value for the CustomerID column ------------------
INSERT INTO Sales.Customers (CustomerID,Company,ContactName,CountryRegion)
     VALUES
           (NEWID(), N'SQL Studios',N'Mickey', N'CA'),
		   (NEWID(), N'SQL Studios', N'Minnie', N'PA'),
		   (NEWID(), N'SQL Management', N'Donald', N'UK'),
		   (NEWID(), N'SQL Studios', N'Daisy', N'UK'),
		   (NEWID(), N'SQL Management', N'Grumpy', N'WA'),
		   (NEWID(), N'SQL Studios', N'Happy', N'PA'),
		   (NEWID(), N'SQL Management', N'Dopey', N'WA');
GO

SELECT *
FROM Sales.Customers;
GO

-----------------------------------------------
--To list PRIMARY KEY Constraints-----------
SELECT *
FROM sys.key_constraints
WHERE type = 'PK';
GO

SELECT * 
FROM sys.objects 
WHERE type = 'PK';
GO

--Displays Metadata information
SELECT * 
FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS 
WHERE CONSTRAINT_TYPE='PRIMARY KEY'

-------------------------------------------------------
DROP TABLE IF EXISTS Sales.Customers;
GO
