/* 70-762 Lesson 6 Lab 6 - Implementing Error Handling in Stored Procedures */
/* Using THROW, RAISERROR, and TRY...CATCH */

/****Using THROW and RAISERROR*****/

-->>>>>>---- Method 1 ----<<<<<<--
-->>>>>> Use THROW anywhere in code
-->>>>>> Invoke THROW with parameters to raise user-defined error message

THROW 51000, 'The stated object cannot be found', 1;
GO

-->>>>>>----Method 2 ----<<<<<<--
-->>>>>> Use THROW within CATCH block
-->>>>>> without parameters to re-raise original error that invoked CATCH block

USE WideWorldImporters;
GO

DROP PROC IF EXISTS Sales.uspBogusOrder;
GO

CREATE PROCEDURE Sales.uspBogusOrder
AS
BEGIN TRY
    UPDATE Sales.OrderLines
	SET Quantity = Quantity/0
		WHERE OrderID = 1;
	SELECT Quantity
	FROM Sales.OrderLines
		WHERE OrderID = 1;
	PRINT 'Completing TRY block';
END TRY
BEGIN CATCH
    PRINT 'Currently beginning catch block.';
	PRINT 'Error Number: ' + CAST(ERROR_NUMBER() AS VARCHAR(50));
    THROW;  -- use the THROW statement to raise the last thrown exception again
END CATCH;
GO

-----------------------------------
EXEC Sales.uspBogusOrder;
----THROW - PRINT statment after THROW will NOT be executed
PRINT 'BogusOrder Procedure Finished';

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
/****USING RAISERROR************************************/
DROP PROC IF EXISTS Sales.uspBogusRaiserrorOrder;
GO

CREATE PROCEDURE Sales.uspBogusRaiserrorOrder
AS
BEGIN TRY
    UPDATE Sales.OrderLines
	SET Quantity = Quantity/0
		WHERE OrderID = 1;
	SELECT Quantity
	FROM Sales.OrderLines
		WHERE OrderID = 1;
	PRINT 'Completing TRY block';
END TRY
BEGIN CATCH
--Get the details of the error
--that invoked the CATCH block
DECLARE
@ErMessage NVARCHAR(2048),
@ErSeverity INT,
@ErState INT

SELECT
@ErMessage = ERROR_MESSAGE(),
@ErSeverity = ERROR_SEVERITY(),
@ErState = ERROR_STATE()

RAISERROR (@ErMessage,
@ErSeverity,
@ErState )
END CATCH;
GO

---------------------------------------
EXEC Sales.uspBogusRaiserrorOrder;
----RAISERROR - PRINT statment after RAISERROR will be executed
PRINT 'BogusRaiserrorOrder Procedure Finished';

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--CAN RAISE user-defined message with message_id greater than 50000
-- which is not defined inSYS.MESSAGES table---------------------

--THROW - Can RAISE user-defined message
THROW 60000, 'This is a user defined message', 1;


--RAISERROR - Can only use messages defined in sys.messages
RAISERROR (60000, 16, 1);
GO
--Now add the Message to SYS.MESSAGES Table by using the below statement:
--EXEC sys.sp_addmessage 60000, 16, This is a user defined message
--and run the code again------------------------------------------------


-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
----In the two examples, the error handling occurs when the procedure is executed
----Using a TRY...CATCH construct


/****EXAMPLE 1****/
USE WideWorldImporters;
GO

-- Verify that the stored procedure does not already exist.    
DROP PROCEDURE IF EXISTS Examples.uspErrorHandlingProc;
GO

-- Create a stored procedure that   
-- generates a divide-by-zero error.  
CREATE OR ALTER PROCEDURE Examples.uspErrorHandlingProc  
AS  
    SELECT 1/0;  
GO  

BEGIN TRY  
    -- Execute the stored procedure inside the TRY block.  
    EXECUTE Examples.uspErrorHandlingProc;  
END TRY  
BEGIN CATCH  
    SELECT	ERROR_PROCEDURE() AS 'Procedure containing Error',
			ERROR_MESSAGE() AS ErrorMessage,
			ERROR_SEVERITY() AS 'Severity of the Error';			  
END CATCH;  
GO


-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>.
/****Example 2****/
USE WideWorldImporters;
GO

-- Verify that the stored procedure does not already exist.    
DROP PROCEDURE IF EXISTS Examples.uspErrorCastProc;
GO

-- Create a stored procedure that   
-- generates a divide-by-zero error.  
CREATE OR ALTER PROCEDURE Examples.uspErrorCastProc 
AS  
    SELECT CAST('This is not an INT' AS INT)
GO  

BEGIN TRY  
    -- Execute the stored procedure inside the TRY block.  
    EXECUTE Examples.uspErrorCastProc;  
END TRY  
BEGIN CATCH  
    SELECT	ERROR_PROCEDURE() AS 'Procedure containing Error',
			ERROR_MESSAGE() AS ErrorMessage,
			ERROR_SEVERITY() AS 'Severity of the Error',
			ERROR_NUMBER() AS 'Error Number',
			ERROR_STATE() AS ErrorState			  
END CATCH;  
GO
