/* 70-762 Lesson 6 Lab 7 - Using Transaction Control Logic ithin Stored Procedures */

USE WideWorldImporters;
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--TRANSACTION INSIDE STORED PROCEDURE-----------

/****Example 1****/
-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
---The following example creates a procedure with
---a transaction included within the procedure
USE WideWorldImporters;
GO
--If Examples Schema does not exist, create it.
--CREATE SCHEMA Examples;
--GO

DROP TABLE IF EXISTS Examples.ProcTranExampleTable;
GO

DROP PROCEDURE IF EXISTS Examples.uspProcTranExample;
GO

CREATE TABLE Examples.ProcTranExampleTable (value int)
GO

-- This stored procedure will roll back a transaction if the 
-- @Rollbk parameter is a 1.
CREATE OR ALTER PROCEDURE Examples.uspProcTranExample @Value int, @Rollbk bit
AS
BEGIN
BEGIN TRANSACTION
INSERT INTO Examples.ProcTranExampleTable VALUES (@Value)
PRINT @@TRANCOUNT;
	IF @Rollbk = 1 
-- If procedure called from within transaction,
-- @@TRANCOUNT when exit procedure different than when started
ROLLBACK TRANSACTION
ELSE
COMMIT
END
GO

--------------------------
-- Begin a new transaction
PRINT @@TRANCOUNT;
BEGIN TRANSACTION
PRINT @@TRANCOUNT;
INSERT INTO Examples.ProcTranExampleTable VALUES (1)
-- Run the sp with the param to roll back a transaction
-- This will return an error because tran count has changed
PRINT @@TRANCOUNT;
EXEC Examples.uspProcTranExample 5,1
PRINT @@TRANCOUNT;
-- Run the commit to close the initial transaction
-- This will return an error because there is no valid transaction
-- to commit
COMMIT
PRINT @@TRANCOUNT;
-- No rows are in the table because the initial insert
-- rolled back
SELECT * FROM Examples.ProcTranExampleTable; --No values in table, everything in stored procedure rolled back
PRINT @@TRANCOUNT;
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--TRANSACTION INSIDE STORED PROCEDURE USING SAVEPOINT-----------
/****Example 2****/
-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
-- This stored procedure will roll back a saved transaction if 
-- the @ROLLBK parameter is a 1.
ALTER PROCEDURE Examples.uspProcTranExample @Value int, @RollBk bit
AS
BEGIN
BEGIN TRANSACTION
SAVE TRANSACTION TranExample --savepoint created
INSERT INTO Examples.ProcTranExampleTable VALUES (@Value)
	PRINT @@TRANCOUNT;
IF @Rollbk = 1 
-- Roll back to the saved point.
-- The transaction is not closed and 
-- @@TRANCOUNT is not changed.
ROLLBACK TRANSACTION TranExample --transaction rolls back to save point
-- Close the transaction created at the beginning of the SP
COMMIT
END
GO

------------------------------
TRUNCATE TABLE Examples.ProcTranExampleTable;
GO
-------------------------------
-- Begin a new transaction
PRINT @@TRANCOUNT;
BEGIN TRANSACTION
PRINT @@TRANCOUNT;
INSERT INTO Examples.ProcTranExampleTable VALUES (1) --this value of 1 is retained in the table, inserted before savepoint
-- Run the sp with the param to roll back a transaction
PRINT @@TRANCOUNT;
EXEC Examples.uspProcTranExample 5,1
PRINT @@TRANCOUNT;
-- Run the commit to close the initial transaction
COMMIT
PRINT @@TRANCOUNT;
SELECT * FROM Examples.ProcTranExampleTable;
PRINT @@TRANCOUNT;
GO

--IN THIS EXAMPLE WE WILL SET THE ROLLBACK VALUE TO 0 SO ALL WORK COMPLETES
-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--TRANSACTION INSIDE STORED PROCEDURE USING SAVEPOINT-----------
/****Example 3****/
-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
-- This stored procedure will roll back a saved transaction if 
-- the @ROLLBK parameter is a 1.
ALTER PROCEDURE Examples.uspProcTranExample @Value int, @RollBk bit
AS
BEGIN
BEGIN TRANSACTION
SAVE TRANSACTION TranExample --savepoint created
INSERT INTO Examples.ProcTranExampleTable VALUES (@Value)
	PRINT @@TRANCOUNT;
IF @Rollbk = 1 
-- Roll back to the savepoint
-- The transaction is not closed and 
-- @@TRANCOUNT is not changed
ROLLBACK TRANSACTION TranExample --transaction rolls back to savepoint
-- Close the transaction created at the beginning of the SP
COMMIT
END
GO

--------------
TRUNCATE TABLE Examples.ProcTranExampleTable;

--------------
-- Begin a new transaction
PRINT @@TRANCOUNT;
BEGIN TRANSACTION
PRINT @@TRANCOUNT;
INSERT INTO Examples.ProcTranExampleTable VALUES (1)
-- Run the sp with the param to roll back a transaction
PRINT @@TRANCOUNT;
EXEC Examples.uspProcTranExample 5,0 --@ROLLBK has value of 0
PRINT @@TRANCOUNT;           -- @@TRANCOUNT decreases by 1 and is now 1
-- Run the commit to close the initial transaction
COMMIT
PRINT @@TRANCOUNT;           --@@TRANCOUNT decreases by 1 and is now 0
SELECT * FROM Examples.ProcTranExampleTable;
PRINT @@TRANCOUNT;			 --Two values in table, first from insert statement and second from proc
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
-->>>>>>>>TRANSACTION CONTROL AND ERROR HANDLING IN STORED PROC<<<<<<<

USE WideWorldImporters;
GO

-- Verify that the stored procedure does not already exist.    
DROP PROCEDURE IF EXISTS Sales.uspTranINProc;
GO

-- Create a stored procedure that   
-- generates a divide-by-zero error.  
CREATE OR ALTER PROCEDURE Sales.uspTranINProc  
AS 
  SET XACT_ABORT, NOCOUNT ON;
  BEGIN TRY
	BEGIN TRANSACTION; 
      SELECT 1/0;
	COMMIT TRANSACTION;  
  END TRY  
  BEGIN CATCH  
      SELECT	ERROR_PROCEDURE() AS 'Procedure containing Error',
			    ERROR_MESSAGE() AS ErrorMessage,
			    ERROR_SEVERITY() AS 'Severity of the Error';
	  IF @@TRANCOUNT > 0
	     ROLLBACK TRANSACTION;
	  THROW;			  
  END CATCH;
  GO
  ------------EXECUTE STORED PROCEDURE---------------------
  EXEC Sales.uspTranINProc;
  GO

SET XACT_ABORT OFF;

------------------------------------------------------------
/* Clean up code if desired */
DROP PROCEDURE IF EXISTS Sales.uspTranINProc;
GO
DROP PROCEDURE IF EXISTS Examples.uspProcTranExample;
GO
DROP TABLE IF EXISTS Examples.ProcTranExampleTable;
GO

