/*  70-762 Lesson 1 Inline Code */


--AFTER TRIGGER------------------------------------------------
USE WideWorldImporters;
GO

DROP TRIGGER IF EXISTS Sales.trCustomerReminder;
GO

CREATE TRIGGER Sales.trCustomerReminder
ON Sales.Customers
AFTER INSERT, UPDATE 
AS RAISERROR ('Notify Customer Relations of Updated Information', 16, 10);
GO


DROP TRIGGER IF EXISTS Sales.trCustomerReminderEmail;
GO

CREATE TRIGGER Sales.trCustomerReminderEmail
ON Sales.Customers
AFTER INSERT, UPDATE, DELETE 
AS
   EXEC msdb.dbo.sp_send_dbmail
        @profile_name = 'WideWorldImporters Administrator',
        @recipients = 'marilyn@WideWorldImporters.com',
        @body = 'Don''t forget to print a report for the sales force.',
        @subject = 'Reminder';
GO

--INSTEAD OF TRIGGER-------------------------------------
USE CertLiveDB
GO
/* Delete Trigger if already created in table */

DROP TRIGGER IF EXISTS Production.trCategories_CategoryNameCheck;
GO

/* CREATE INSTEAD OF Trigger */
CREATE TRIGGER Production.trCategories_CategoryNameCheck
ON Production.Categories
INSTEAD OF INSERT
AS
BEGIN
  SET NOCOUNT ON;            --count of number of rows affected not returned --
  IF EXISTS (SELECT COUNT(*)
        FROM Inserted AS I
        JOIN Production.Categories AS C
          ON I.CategoryName = C.CategoryName
		GROUP BY I.CategoryName
		HAVING COUNT(*) > 0 )  --check if Inserted CategoryName is already in Production.Categories --
		THROW 50000, 'Duplicate category names are not allowed', 0; --display error message --
	ELSE
	BEGIN
		INSERT Production.Categories (CategoryName, Description) -- if not duplicate INSERT the category name --
		SELECT CategoryName, Description FROM Inserted;
	END;
END;
GO

--First Display Existing Categories
SELECT *
FROM Production.Categories;
GO

/* Test the INSTEAD OF Trigger  - INSERT a category name That already exists in the table */
INSERT INTO Production.Categories (CategoryName,Description)
     VALUES ('SQL 2016 Database Administration', 'New Category Description');
GO

/* Test the INSTEAD OF Trigger - INSERT a new category name the table */
INSERT INTO Production.Categories (CategoryName,Description)
     VALUES ('SQL 2016 BI Development', 'Microsoft Solutions Associate');
GO

/* Cleanup by deleting Trigger */

DROP TRIGGER IF EXISTS Production.trCategories_CategoryNameCheck;
GO

-- Use Trigger Update Functions---------------------------------
USE CertLiveDB;
GO

-- Example of DML Trigger Function referenced by an INSERT or UPDATE statement --

IF UPDATE(Price)
BEGIN    
	PRINT 'Column Price is being updated'    
-- do the required processing here 
END;

--The UPDATE statement below would make the UPDATE(Price) Function True --
UPDATE Production.Products
	SET Price = 200
	WHERE ProductID = 761;